=== modified file 'MANIFEST.in'
--- MANIFEST.in	2011-03-10 14:22:12 +0000
+++ MANIFEST.in	2012-02-02 04:58:46 +0000
@@ -1,2 +1,6 @@
-include README.txt NEWS
+include NEWS
+include README.txt
+include apache-loggerhead.conf
+include bazaar.conf
+include loggerhead.conf.example
 recursive-include loggerhead/static *

=== modified file 'NEWS'
--- NEWS	2011-11-08 16:29:37 +0000
+++ NEWS	2012-02-02 04:58:46 +0000
@@ -1,8 +1,8 @@
 What's changed in loggerhead?
 =============================
 
-1.19 [???]
-----------------
+1.19 [????]
+-----------
 
     - Add ``bzr load-test-loggerhead`` as a way to make sure loggerhead can
       handle concurrent requests, etc. Scripts can be written that spawn
@@ -49,6 +49,9 @@
       marked as private and a notification ribbon to be displayed via
       javascript on the view pages. (JC Sackett, #823471)
 
+    - Add a script and documentation for running under mod_wsgi.
+      (Stuart Colville, Toshio Kuratomi)
+
 
 1.18.1 [24Mar2011]
 ------------------
@@ -94,7 +97,6 @@
     - Fix bad redirect when visiting "/download" or "/download/".
       (Matt Nordhoff, #247992)
 
-
 1.17 [20Aug2009]
 ----------------
 

=== added file 'apache-loggerhead.conf'
--- apache-loggerhead.conf	1970-01-01 00:00:00 +0000
+++ apache-loggerhead.conf	2010-04-18 17:55:33 +0000
@@ -0,0 +1,19 @@
+### If you receive MemoryError tracebacks setting up loggerhead under mod_wsgi,
+### read /usr/share/doc/loggerhead*/README for help
+Alias /bzr/static /usr/share/loggerhead/static
+RewriteEngine On
+RewriteRule ^/bzr$ /bzr/ [R]
+
+WSGIDaemonProcess loggerhead user=apache group=apache maximum-requests=1000 display-name=loggerhead processes=4 threads=1
+WSGISocketPrefix run/wsgi
+WSGIRestrictStdout On
+WSGIRestrictSignal Off
+
+WSGIScriptAlias /bzr /usr/bin/loggerhead.wsgi
+
+<Location /bzr>
+    WSGIProcessGroup loggerhead
+    Order deny,allow
+    Allow from all
+</Location>
+

=== added file 'bazaar.conf'
--- bazaar.conf	1970-01-01 00:00:00 +0000
+++ bazaar.conf	2010-04-18 14:41:23 +0000
@@ -0,0 +1,9 @@
+# directory to serve bzr branches from
+# Non-bzr directories under this path will also be visible in loggerhead
+#http_root_dir = '/var/www/bzr'
+
+# The url prefix for the bzr branches.
+http_user_prefix = '/bzr'
+
+# Directory to put cache files in
+http_sql_dir = '/var/cache/loggerhead'

=== modified file 'docs/index.rst'
--- docs/index.rst	2010-05-14 07:33:29 +0000
+++ docs/index.rst	2012-02-02 04:58:46 +0000
@@ -149,6 +149,61 @@
 some path into the site, you'll need to specify it using
 ``--prefix=/some_path``.
 
+Serving Loggerhead with mod_wsgi
+--------------------------------
+
+A second method for using Loggerhead with apache is to have apache itself
+execute Loggerhead via mod_wsgi.  You need to add configuration for apache and
+for bazaar to make this work.  Example config files are in the Loggerhead doc
+directory as apache-loggerhead.conf and bazaar.conf.  You can copy them into
+place and use them as a starting point following these directions:
+
+1) Install mod_wsgi.  On Ubuntu and other Debian derived distros::
+
+    sudo apt-get install libapache2-mod-wsgi
+
+   On Fedora-derived distros::
+
+    su -c yum install mod_wsgi
+
+2) Copy the bazaar.conf file where apache will find it (May be done for you if
+   you installed Loggerhead from a distribution package)::
+
+    # install -d -o apache -g apache -m 0755 /etc/loggerhead
+    # cp -p /usr/share/doc/loggerhead*/bazaar.conf /etc/loggerhead/
+    # ln -s /etc/loggerhead /var/www/.bazaar
+
+3) Create the cache directory (May be done for you if you installed Loggerhead
+   from a distribution package)::
+
+    # install -d -o apache -g apache -m 0700 /var/cache/loggerhead/
+
+4) Edit /etc/loggerhead/bazaar.conf.  You need to set http_root_dir to the filesystem
+   path that you will find your bzr branches under.  Note that normal
+   directories under that path will also be visible in Loggerhead.
+
+5) Install the apache conf file::
+
+     # cp -p /usr/share/doc/loggerhead*/apache-loggerhead.conf /etc/httpd/conf.d/loggerhead.conf
+
+6) Edit /etc/httpd/conf.d/loggerhead.conf to point to the url you desire to
+   serve Loggerhead on.  This should match with the setting for
+   http_user_prefix in bazaar.conf
+
+7) Restart apache and you should be able to start browsing
+
+.. note:: If you have SELinux enabled on your system you may need to allow
+   apache to execute files in temporary directories.  You will get a
+   MemoryError traceback from python if this is the case.  This is because of
+   the way that python ctypes interacts with libffi.  To rectify this, you may
+   have to do several things, such as mounting tmpdirs so programs can be
+   executed on them and setting this SELinux boolean::
+
+       setsebool httpd_tmp_exec on
+
+   This bug has information about how python and/or Linux distros might solve
+   this issue permanently and links to bugs which diagnose the root cause.
+   https://bugzilla.redhat.com/show_bug.cgi?id=582009
 
 Search
 ------

=== added file 'loggerhead.wsgi'
--- loggerhead.wsgi	1970-01-01 00:00:00 +0000
+++ loggerhead.wsgi	2010-04-18 17:58:04 +0000
@@ -0,0 +1,50 @@
+#!/usr/bin/python -tt
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import sys
+import os
+import pwd
+sys.path.insert(0, os.path.dirname(__file__))
+
+from paste.httpexceptions import HTTPExceptionHandler
+from loggerhead.apps.transport import BranchesFromTransportRoot
+from loggerhead.apps.error import ErrorHandlerApp
+from loggerhead.config import LoggerheadConfig
+from bzrlib import config as bzrconfig
+from paste.deploy.config import PrefixMiddleware
+from bzrlib.plugin import load_plugins
+
+class NotConfiguredError(Exception):
+    pass
+
+
+load_plugins()
+config = LoggerheadConfig()
+prefix = config.get_option('user_prefix') or ''
+# Note we could use LoggerheadConfig here if it didn't fail when a
+# config option is not also a commandline option
+root_dir = bzrconfig.GlobalConfig().get_user_option('http_root_dir')
+if not root_dir:
+    raise NotConfiguredError('You must have a ~/.bazaar/bazaar.conf file for'
+            ' %(user)s with http_root_dir set to the base directory you want'
+            ' to serve bazaar repositories from' %
+            {'user': pwd.getpwuid(os.geteuid()).pw_name})
+prefix = prefix.encode('utf-8', 'ignore')
+root_dir = root_dir.encode('utf-8', 'ignore')
+app = BranchesFromTransportRoot(root_dir, config)
+app = PrefixMiddleware(app, prefix=prefix)
+app = HTTPExceptionHandler(app)
+application = ErrorHandlerApp(app)

=== modified file 'setup.py'
--- setup.py	2011-03-10 14:22:12 +0000
+++ setup.py	2012-02-02 04:58:46 +0000
@@ -30,7 +30,10 @@
     license = "GNU GPL v2 or later",
     maintainer = "Michael Hudson",
     maintainer_email = "michael.hudson@canonical.com",
-    scripts = ["serve-branches"],
+    scripts = [
+        "serve-branches",
+        "loggerhead.wsgi",
+        ],
     packages = ["loggerhead",
                 "loggerhead/apps",
                 "loggerhead/controllers",
@@ -65,5 +68,8 @@
                                    "static/images/*"]},
     data_files = [
         ('share/man/man1', ['serve-branches.1']),
+        ('share/doc/loggerhead', ['loggerhead.conf.example',
+                                  'apache-loggerhead.conf',
+                                  'bazaar.conf']),
         ],
     )

